import React, { useState } from 'react';

import styles from '../styles/Input.scss'

/**
	 * 	@param  {Object} props Objeto compuesto por los siguientes parámetros
	 * 		@param {String} props.type Tipo del input, algunos de ellos incluyen validaciones
	 * 		@param {String} props.style Estilos por defecto del proyecto que tendrá cada uno de los input
	 * 		@param {String} props.placeHolder Placeholder que parece dentro del input
	 * 		@param {Boolean} props.required Indica si el campo es obligatorio
	 * 		@param {String} props.name Nombre del input
	 * 		@param {String} [props.value] Valor que tiene el input por defecto
	 * 		@param {String} [props.size = md] Altura del input, tres tamaños (sm -> 2.5; md -> 3rem; lg-> 3.5rem)
	 * 
*/

const Input = props => {
	const [state, setState] = useState({ advise: '', borderStyle: '', checkedStyle: 'none' });
	const [focusStyle, placeStyle, ctnStyle, inputStyle] = getStyles(props.size);
	const [type, changeHandler] = getType(props.type);
	const style = props.style ? props.style : "";
	const placeHolder = props.required ? props.placeHolder + ' *' : props.placeHolder;
	const onChange = props.onChange ? props.onChange : null;

	return (
		<div styleName='input-ctn'>
			<div
				styleName={`${ctnStyle} ${state.borderStyle} wrapper`}
				className={style}>
				<input
					className={`${!props.value ? focusStyle : ""}`}
					styleName={inputStyle}
					name={props.name}
					defaultValue={props.value}
					type={type}
					onBlur={ev => changeHandler(ev, focusStyle, placeStyle, setState)}
					onChange={onChange}
					onKeyPress={props.disableEnter}>
				</input>
				<p className={`${!props.value ? "" : `${styles.nonEmpty} ${placeStyle}`}`}>
					{placeHolder}
				</p>
				<span styleName={state.checkedStyle} />
			</div>
			{state.advise}
		</div>
	)
};

const getStyles = size => {
	let focusStyle, placeStyle, ctnStyle, inputStyle;

	switch (size) {
		case 'sm':
			ctnStyle = 'ctnSm';
			inputStyle = 'inputSm';
			focusStyle = styles.focusSm;
			placeStyle = styles.placeSm;
			break;
		case 'md':
			ctnStyle = 'ctnMd';
			inputStyle = 'inputMd';
			focusStyle = styles.focusMd;
			placeStyle = styles.placeMd;
			break;
		case 'lg':
			ctnStyle = 'ctnLg';
			inputStyle = 'inputLg';
			focusStyle = styles.focusLg;
			placeStyle = styles.placeLg;
			break;
		default:
			ctnStyle = 'ctnMd';
			inputStyle = 'inputMd';
			focusStyle = styles.focusMd;
			placeStyle = styles.placeMd;
	}

	return [focusStyle, placeStyle, ctnStyle, inputStyle];

};

const getType = selectedType => {
	let type, changeHandler;

	switch (selectedType) {
		case 'name':
			type = 'text';
			changeHandler = nameHandler;
			break;
		case 'zip':
			type = 'text';
			changeHandler = zipHandler;
			break;
		case 'mail':
			type = 'text';
			changeHandler = mailHandler;
			break;
		case 'pass':
			type = 'password';
			changeHandler = passHandler;
			break;
		case 'tel':
			type = 'text';
			changeHandler = telHandler;
			break;
		case 'nif':
			type = 'text'
			changeHandler = nifHandler;
			break;
		case 'text':
		default:
			type = 'text';
			changeHandler = inputBlur;
			break;
	}

	return [type, changeHandler];
};

const inputBlur = (ev, focusStyle, placeStyle) => {
	const element = ev.target;
	const placeHolder = element.parentElement.children[1];

	if (element.value) {
		element.classList.remove(focusStyle);
		placeHolder.classList.add(styles.nonEmpty, placeStyle);
	}
	else {
		placeHolder.classList.remove(styles.nonEmpty, placeStyle);
		element.classList.add(focusStyle);
	}
};

const nameHandler = (ev, focusStyle, placeStyle, setState) => {
	inputBlur(ev, focusStyle, placeStyle);

	const name = ev.target.value.replace(/[\s-]+/g, '');
	const valExp = /^[A-Za-zÀ-ÿ\u00f1\u00d1\u00c7\u00e7}\s]+$/;

	if (name.length < 3) {
		setState({
			advise: <p>*Introduce al menos 2 caracteres</p>,
			borderStyle: 'borderRed',
			checkedStyle: 'none'
		});
	} else if (!name.match(valExp)) {
		setState({
			advise: <p>*Introduce sólo letras</p>,
			borderStyle: 'borderRed',
			checkedStyle: 'none'
		});
	} else {
		setState({
			advise: '',
			borderStyle: '',
			checkedStyle: ''
		});
	}
};

const zipHandler = (ev, focusStyle, placeStyle, setState) => {
	inputBlur(ev, focusStyle, placeStyle);

	const zip = ev.target.value.toUpperCase().replace(/[\s-]+/g, '');
	const valExp = /^[0-9]+$/;
	if (zip.length != 5 || zip < 1000 || zip > 52999 || !zip.match(valExp)) {
		setState({
			advise: <p>*Introduce un código postal válido</p>,
			borderStyle: 'borderRed'
		});
	} else {
		setState({
			advise: '',
			borderStyle: ''
		});
	}
};

const mailHandler = (ev, focusStyle, placeStyle, setState) => {
	inputBlur(ev, focusStyle, placeStyle);

	const mail = ev.target.value.replace(/[\s-]+/g, '');
	const valExp = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
	if (!mail.match(valExp)) {
		setState({
			advise: <p>*Introduce un correo válido</p>,
			borderStyle: 'borderRed'
		});
	} else {
		setState({
			advise: '',
			borderStyle: ''
		});
	}
};

const passHandler = (ev, focusStyle, placeStyle, setState) => {
	inputBlur(ev, focusStyle, placeStyle);

	const pass = ev.target.value.replace(/[\s-]+/g, '');
	const valExp = /^[\x21-\x7E]+$/;
	if (pass.length < 6) {
		setState({
			advise: <p>*La contraseña debe contener al menos 6 caracteres</p>,
			borderStyle: 'borderRed'
		});
	} else if (!pass.match(valExp)) {
		setState({
			advise: <p>*La contraseña contiene caracteres no válidos</p>,
			borderStyle: 'borderRed'
		});
	} else {
		setState({
			advise: '',
			borderStyle: ''
		});
	}
};

const telHandler = (ev, focusStyle, placeStyle, setState) => {
	inputBlur(ev, focusStyle, placeStyle);

	const tel = ev.target.value.replace(/[\s-]+/g, '');
	const valExp = /^[6789]\d+$/;
	if (tel.length != 9 || !tel.match(valExp)) {
		setState({
			advise: <p>*Introduce un teléfono válido</p>,
			borderStyle: 'borderRed'
		});
	} else {
		setState({
			advise: '',
			borderStyle: ''
		});
	}
};

const nifHandler = (ev, focusStyle, placeStyle, setState) => {
	inputBlur(ev, focusStyle, placeStyle);

	const nif = ev.target.value.toUpperCase().replace(/[\s-]+/g, '');
	const valExpNif = /^(\d|[XYZ])\d{7}[A-Z]$/;
	const valExpCif = /^[ABCDEFGHJKLMNPQRSUVW]\d{7}[\dA-J]$/;
	let flag = true;

	if (valExpNif.test(nif)) {
		let num = nif.match(/\d+/);
		num = (nif[0] != 'Z' ? nif[0] != 'Y' ? 0 : 1 : 2) + num;
		if (nif[8] == 'TRWAGMYFPDXBNJZSQVHLCKE'[num % 23]) {
			console.log(/^\d/.test(nif) ? `DNI ${nif}` : `NIE ${nif}`);
			flag = false;
			setState({
				advise: '',
				borderStyle: ''
			});
		}
	}
	else if (valExpCif.test(nif)) {
		for (var sum = 0, i = 1; i < 8; ++i) {
			let num = nif[i] << i % 2;
			let uni = num % 10;
			sum += (num - uni) / 10 + uni;
		}
		var c = (10 - sum % 10) % 10;
		if (nif[8] == c || nif[8] == 'JABCDEFGHI'[c]) {
			let out = /^[KLM]/.test(nif) ? `NIF ${nif}` : `NIF ${nif}`;
			console.log(out);
			flag = false;
			setState({
				advise: '',
				borderStyle: ''
			});
		}
	}

	if (flag) {
		setState({
			advise: <p>*Introduce un NIF, DNI o NIE válido</p>,
			borderStyle: 'borderRed'
		});
	}
};

export default Input;